// ==UserScript==
// @name           NicoVideo Additional MyList (for SeaHorse)
// @namespace      http://castor.s26.xrea.com/
// @description    Save the movie list to local settings.
// @include        http://www.nicovideo.jp/watch/*
// @include        http://www.nicovideo.jp/my
// @type           SleipnirScript
// @version        0.9.8.20090625sh
// @author         castor <castor.4bit@gmail.com>
// ==/UserScript==

var ScriptControl = CreateObject("ScriptControl");
if (ScriptControl && !_window[ScriptName]) {
    _window[ScriptName] = true;
    ScriptControl.Language = "JScript";
    ScriptControl.AddObject("ScriptControl", ScriptControl);
    ScriptControl.AddObject("sleipnir", sleipnir, true);
    ScriptControl.AddObject("db", Database.Open("NvAddMylist"));
    ScriptControl.AddObject("_window", _window);
    ScriptControl.Eval("_window.attachEvent('onunload', function(){ScriptControl = null;})");
    ScriptControl.AddCode(NvAddMyList);
    ScriptControl.Run("NvAddMyList", _window);
}

function NvAddMyList(window) {
    //===============================================================
    var NUM_DISPALY     = 5;		// number of display per page
    var OPEN_NEW_WINDOW = true;		// open new window
    
    
    //===============================================================
    var w = window, document = w.document;
    var tm;		                // reload timer
    var handler = new Array;    // event handler
    
    var MyList = function() {
        return MyList.prototype.getInstance();
    }
    MyList.prototype = {
        _inst: null,
        items: null,
        tags:  null,
        pos:     0,
        showtag: 0,
        key_list: 'additional_mylist',
        key_tags: 'additional_mylist_tags',
        
        //-------------------------------------------------
        getInstance: function() {
            if (!MyList.prototype._inst) {
                this.loadList();
                this.loadTags();
                MyList.prototype._inst = this;
            }
            return MyList.prototype._inst;
        },
        
        loadList: function() {
        	this.items = this._load(this.key_list);
        },
        loadTags: function() {
        	this.tags = this._load(this.key_tags);
        },
        
        saveList: function() {
        	this._save(this.key_list, this.items);
        },
        saveTags: function() {
        	this._save(this.key_tags, this.tags);
        },
        
        findItem: function(id) {
            this.loadList();
            return this._find(id, this.items);
        },
        addItem: function(item) {
        	this.loadList();
        	if (this._add(item, this.items)) {
        		this.saveList();
        		return true;
        	}
        	return false;
        },
        deleteItem: function(item) {
        	this.loadList();
        	
        	var len = item.tags.length;
        	for (var i=0; i<len; ++i) {				// delete item tags
        		this.deleteItemTag(item.id, item.tags[i]);
        	}
        	
        	if (this._delete(item, this.items)) {	// delete item
        		this.saveList();
                return true;
        	}
        	return false;
        },
        
        addTag: function(tid, str) {
        	this.loadTags();
        	
        	if (str.length == 0) {	// check string length
        		GM_log('invalid tag name.');
        		return false;
        	}
        	
        	if (this._add(new TagItem(tid, str), this.tags)) {
        		this.saveTags();
        		return true;
        	}
        	return false;
        },
        addItemTag: function(id, str) {
        	var idx = this._find(id, this.items);
        	var max = 100;
        	var tid = -1;
        	var len;
        	
        	len = this.tags.length;
        	for (var i=0; i<len; ++i) {
        		if (max < this.tags[i].id) {
        			max = this.tags[i].id;
        		}
        		if (str == this.tags[i].t) {	// check registered
        			tid = this.tags[i].id;
        			break;
        		}
        	}
        	
        	if (tid < 0) {
        		tid = max + 1;	// set new tag id (current max value + 1)
        		if (!this.addTag(tid, str)) return false;
        	}
        	
        	len = this.items[idx].tags.length;
        	for (var i=0; i<len; ++i) {
        		if (tid == this.items[idx].tags[i]) {
        			return false;
        		}
        	}
        	
        	this.items[idx].tags.push(tid);
    		this.saveList();
    		
    		return true;
        },
        
        deleteTag: function(item) {
        	this.loadTags();
        	
        	if (this._delete(item, this.tags)) {
        		this.saveTags();
                return true;
        	}
        	return false;
        },
        deleteItemTag: function(id, tid) {
        	var cnt = 0;
        	var len = this.items.length;
        	var tlen;
        	
        	this.loadList();
        	for (var i=0; i<len; ++i) {
        		tlen = this.items[i].tags.length;
        		
        		for (var j=0; j<tlen; ++j) {
        			if (tid == this.items[i].tags[j]) {
        				if (id == this.items[i].id) {	// delete from list item
		        			this.items[i].tags.splice(j, 1);
		        			this.saveList();
		        		} else {		// count tag reference
		        			cnt++;
		        		}
		        		break;
		        	}
		        }
		    }
		    
		    if (cnt == 0) {
		    	this.loadTags();
		    	var idx = this._find(tid, this.tags);
		    	if (idx === false) {
		    		return false;
		    	}
	    	    this.tags.splice(idx, 1);
		        this.saveTags();
	        }
	        return true;
        },
        
        getTagName: function(id) {
        	var len = this.tags.length;
        	for (var i=0; i<len; ++i) {
                if (id == this.tags[i].id) {
                    return this.tags[i].t;
                }
            }
        	return false;
        },
        
        _load: function(key, data) {
            var value = GM_getValue(key);
            
            if (typeof value == 'undefined') {
            	return new Array();
            } else {
            	return eval('('+ value +')');
            }
        },
        _save: function(key, data) {
            GM_setValue(key, uneval(data));
        },
        
        _add: function(item, data) {
            if (this._find(item.id, data) === false) {
                data.push(item);
                return true;
            }
            return false;
        },
        _delete: function(item, data) {
            var idx = this._find(item.id, data);
            if (idx !== false) {
                data.splice(idx, 1);
                return true;
            }
            return false;
        },
        
        _find: function(id, data) {
        	var len = data.length;
            for (var i=0; i<len; ++i) {
                if (id == data[i].id) {
                    return i;
                }
            }
            return false;
        }
    }
    
    var ListItem = function(id, v, title) {
        this.id = id;               // id (watch/XXXXXXXX)
        this.v = v;                 // id (smXXXXXXXX)
        this.t = title;             // title
        this.c = '';                // comment
        this.tags = new Array();    // tags
        this.reg = new Date();      // registered date
    }
    
    var TagItem = function(id, tag) {
        this.id = id;               // id
        this.t  = tag;              // tag name
    }
    
    //===============================================================
    function screenWatch() {
        var mylist_add_submit = document.getElementById('mylist_add_submit');
        
        if (mylist_add_submit) {
            var input  = document.createElement('input');
            input.className = 'submit';
            input.setAttribute('id',   'mylist_add_button_add');
            input.setAttribute('type', 'button');
            input.setAttribute('value', 'gXgɓo^');
            input.style.cssText = 'margin-left:4px;';
            input.attachEvent('onclick', addMyList);
            
            if (!document.getElementById('mylist_add_button_add')) {
                mylist_add_submit.parentNode.appendChild(input);
                mylist_add_submit.parentNode.style.cssText = 'white-space:nowrap;';
            }
        }
        
        //-----------------------------------------------------------
	    function addMyList() {
	        var item = getVideoInfo();
	        
	        if (item) {
	            if (mylist.addItem(item)) {
	                setMessage("g}CXgɓo^܂()B");
	            } else {
	                setMessage("łɓo^Ă܂B");
	            }
	        } else {
	            setMessage("o^Ɏs܂corz");
	        }
	    }
	    
	    function setMessage(str) {
	        w.balloon('mylist_add_submit', str, 'lT');
	    }
    }
    
    //===============================================================
    function screenMylist() {
        // Additional List (Header)
        var div  = document.createElement('div');
        
        div.setAttribute('id', 'mylist_additional');
        div.setAttribute('width', '100%');
        div.appendChild(makeListHeader());		// header
        div.appendChild(makeListSubHeader());	// sub-header
        div.appendChild(makeListBody());		// body
        
        var pos = 0;
        var a = document.getElementsByTagName('div');
        for (var i=0; i<a.length; ++i) {
            if (a[i].getAttribute('className') == 'tool_tit_bg') {
                if (++pos == 2) {
                    var m = a[i];
                    m.parentNode.insertBefore(div, m);
                    break;
                }
            }
        }
        /*
        var a = document.getElementsByTagName('table');
        for (var i=0; i<a.length; ++i) {
            if (a[i].getAttribute('summary', 2) == '}CXg') {
                var m = a[i];
                m.parentNode.insertBefore(div, m.nextSibling);
                break;
            }
        }
        */
        
        updateTagSelectBox('mylist_select_tag', true);	// Update tag select box
        updateList();		// Update List
        setReloadTimer();	// reload at regular intervals
    }
    
    //-----------------------------------------------------
    function setReloadTimer() {
    	if (tm == null) {
	    	tm = w.setInterval(function () {
	            mylist.loadList();
	            mylist.loadTags();
	            updateList();
	        }, 15000);
	    }
    }
    
    //-----------------------------------------------------
    function updateList() {
        var rlist  = ([].concat(mylist.items)).reverse();
        var target = document.getElementById('mylist_additional_body');
        var limit  = NUM_DISPALY;
        
        for (var key in handler) {
            detachHandler(key, 'onclick');
        }
        removeAllChild(target);				// Clear current list
        updateItemCount(rlist.length);		// Update item count
        
        // tag filtering
        if (mylist.showtag > 0) {
        	var _sel = document.getElementById('mylist_select_tag');
            var _tid = _sel[_sel.selectedIndex].value;
            
        	var _len = rlist.length;
        	var _tlen;
        	var _del;
        	for (var i=_len-1; i>=0; --i) {
        		_tlen = rlist[i].tags.length;
        		_del  = true;
        		for (var j=0; j<_tlen; ++j) {
        			if (_tid == rlist[i].tags[j]) {
        				_del = false;
                        break;
        			}
        		}
        		if (_del) {
                	rlist.splice(i, 1);
                }
        	}
        }
        
        if (rlist.length > 0) {
        	// Redraw item list
        	var len = rlist.length;
            for (var i=0; i<len; ++i) {
                if (i < mylist.pos) continue;      // skip offset
                if (limit <= 0)     break;         // show page limit
                
                target.appendChild(makeVideoThumb(rlist[i], --limit));		// video thumbnail
                target.appendChild(makeDotLine());
                updateTagLink(rlist[i]);
            }
            
            // Pagination
            var page_tr = document.createElement('tr');
            var page_td = document.createElement('td');
            
            page_td.colSpan = '4';
            page_td.style.cssText = 'border-bottom:none';
            
            page_td.appendChild(makePagenation(rlist));
            page_tr.appendChild(page_td);
            target.appendChild(page_tr);
            
        } else {
        	// No Items
            var tr   = document.createElement('tr');
            var td   = document.createElement('td');
            var div  = document.createElement('div');
            
            div.innerHTML = 'o^͂܂B';
            div.setAttribute('width', '80%');
            div.className = 'TXT12';
            div.style.cssText = 'border:1px dotted #888888; background-color:#f8f8f8; marginn:0; padding:1em';
            td.setAttribute('width', '100%');
            td.style.cssText = 'text-align: center;';
            td.appendChild(div);
            tr.appendChild(td);
            target.appendChild(tr);
        }
    }
    
    //===============================================================
    function makeListHeader() {
    	var div = document.createElement('div');
        
        div.style.cssText = 'padding:4px 8px';
        div.className = 'tool_tit_bg tool_tit';
        div.innerHTML = 'g}CXg';
    	
    	return div;
    }
    
    function makeListSubHeader() {
    	var table  = document.createElement('table');
        var tbody  = document.createElement('tbody');
        var tr     = document.createElement('tr');
        var td1    = document.createElement('td');
        var td2    = document.createElement('td');
        var tagbox = document.createElement('select');
        
        td1.innerHTML = '搔F&nbsp;<strong id="mylist_items_length">'+ mylist.items.length +'</strong> ( ő假 ) ';
        td1.className = 'TXT12';
        td1.style.cssText = 'vertical-align:top';
        td2.className = 'TXT12';
        td2.style.cssText = 'white-space:nowrap; text-align:right';
        
        tagbox.setAttribute('id', 'mylist_select_tag');
        tagbox.style.cssText = 'border: solid 1px #888;';
        tagbox.attachEvent('onchange', _change_tag);
        
        tbody.setAttribute('id', 'mylist_additional_subheader');
        table.setAttribute('width', '100%');
        table.setAttribute('border', '0');
        table.setAttribute('cellpadding', '4');
        table.setAttribute('cellspacing', '0');
        
        td2.appendChild(tagbox);
        tr.appendChild(td1);
        tr.appendChild(td2);
        tbody.appendChild(tr);
        
        table.appendChild(tbody);
        
        return table;
    }
    
    function makeListBody() {
    	var table = document.createElement('table');
        var tbody = document.createElement('tbody');
        
        table.setAttribute('width', '100%');
        table.setAttribute('border', '0');
        table.setAttribute('cellpadding', '0');
        table.setAttribute('cellspacing', '4');
        table.style.cssText = 'margin-bottom:16px;';
        table.className = 'sima_table';
        tbody.setAttribute('id', 'mylist_additional_body');
        
        table.appendChild(tbody);
    	
    	return table;
    }
    
    //-----------------------------------------------------
    function makeVideoThumb(item, idx) {
        var tr  = document.createElement('tr');
        var td1 = document.createElement('td');
        var td2 = document.createElement('td');
        var td3 = document.createElement('td');
        var td4 = document.createElement('td');
        
        var vid = item.v || item.id;
        var prefix = vid.substring(0, 2);
        var suffix = vid.substring(2);
        
        var img_banner = 'http://res.nicovideo.jp/img/cmn_thumb/from/'+ prefix +'.gif';
        var img_thumb  = 'http://tn-skr.smilevideo.jp/smile?i='+ suffix;
        var img_width  = 62;
        var img_height = 48;
        
        if (vid.indexOf("am") === 0) {
            img_width  = 48;
        }
        var link_target = (OPEN_NEW_WINDOW)? 'target="_blank"' : '';
        
        //-----------------------------------------------------------
        td1.innerHTML = '<p style="margin:4px 2px; width:62px; height:48px">'
                      + '<a href="watch/'+ item.id +'" '+ link_target +'>'
                      + '<img alt="'+ Punycode.decode(item.t) +'" src="'+ img_thumb +'" width="'+ img_width +'" height="'+ img_height +'" style="border:1px solid #333333;" />'
                      + '</a></p>';
        
        td2.innerHTML = '<p><img src="'+ img_banner +'" /></p>'
                      + '<p style="margin-top:4px;" class="TXT12">'
                      + '<strong>'+ getDateFormat(item.reg) +'</strong> o^ &nbsp;&nbsp;'
                      + '<span id="am_tags_'+ item.id +'"></span><br>'
                      + '<a class="video" href="watch/'+ item.id +'" '+ link_target +'>'+ Punycode.decode(item.t) +'</a>'
                      + '</p>'
                      + '<p id="am_desc_'+ item.id +'" class="TXT12" style="word-break: break-all;">'+ escapeString(Punycode.decode(item.c)) +'</p>';
        
        td3.appendChild(makeButton('am_edit_'+ item.id, 'ҏW',  _begin_edit ));
        td4.appendChild(makeButton('am_del_'+  item.id, '폜', _delete_item ));
        
        td2.setAttribute('width', '100%');
        tr.setAttribute('class',  ((idx % 2)? 'odd' : 'even'));
        
        tr.appendChild(td1);
        tr.appendChild(td2);
        tr.appendChild(td3);
        tr.appendChild(td4);
        
        return tr;
    }
    
    //-----------------------------------------------------
    function makeEditPanel(item) {
    	var div_base         = document.createElement('div');		// base
        var div_comment_head = document.createElement('div');		// comment header
        var div_comment      = document.createElement('div');		// comment body
        var div_tags_head    = document.createElement('div');		// tag header
        var div_tags         = document.createElement('div');		// tag body
        var div_footer       = document.createElement('div');		// footer
        
        // base
        div_base.setAttribute('id',    'am_editform_'+ item.id );
        div_base.style.cssText = 'margin:4px; padding: 4px; border:solid 1px #666; background-color:#ddd; display:none;';
        
        // comment header
        div_comment_head.innerHTML = '';
        div_comment_head.style.cssText = 'padding-left:2px; border-bottom:solid 1px #666;';
        div_comment_head.className = 'TXT12';
        
        // comment body
        var comment   = document.createElement('textarea');
        comment.value = escapeString(Punycode.decode(item.c));
        comment.setAttribute('id',    'am_edit_comment_'+ item.id );
        comment.style.cssText = 'width:95%; margin:2px 2px 0.5em 2px; padding: 2px; border:solid 1px #666; font-size:12px';
        div_comment.appendChild(comment);
        
        // tag header
        div_tags_head.innerHTML = '^O';
        div_tags_head.style.cssText = 'padding-left: 2px; border-bottom:solid 1px #666;';
        div_tags_head.className = 'TXT12';
        
        // tag body
        var tag_table = document.createElement('table');
        var tag_tbody = document.createElement('tbody');
        tag_tbody.setAttribute('id',    'am_edit_tags_'+ item.id);
        tag_table.style.cssText = 'margin: 0 0 1em 1em';
        tag_table.appendChild(tag_tbody);
        div_tags.appendChild(tag_table);
        
        // footer
        var footer_table = document.createElement('table');
        var footer_tbody = document.createElement('tbody');
        var footer_tr    = document.createElement('tr');
        var footer_td1   = document.createElement('td');
        var footer_td2   = document.createElement('td');
        
        var input  = document.createElement('input');
        var button = document.createElement('input');
        var submit = document.createElement('input');
        
        input.setAttribute('id',   'am_append_tag_edit_'+ item.id);
        input.setAttribute('type', 'text');
        button.setAttribute('id',    'am_append_tag_input_'+ item.id);
        button.setAttribute('type',  'button');
        button.className = 'submit';
        button.setAttribute('value', '^Oǉ');
        submit.setAttribute('id',    'am_editend_'+ item.id );
        submit.setAttribute('type',  'button');
        submit.className = 'submit';
        submit.style.cssText = 'vertical-align:bottom';
        submit.setAttribute('value', 'ҏWI');
        
        button.attachEvent('onclick', bindFunc(button, _add_tag));
        submit.attachEvent('onclick', bindFunc(submit, _end_edit));
        
        footer_td1.className = 'TXT12';
        footer_td2.className = 'TXT12';
        footer_td1.style.cssText = 'border:none; white-space: nowrap;';
        footer_td2.style.cssText = 'border:none; white-space: nowrap; width:100%; text-align:right';
        
        //footer_td1.appendChild( document.createTextNode('^OǉF') );
        footer_td1.appendChild( input );
        footer_td1.appendChild( document.createTextNode(' ') );
        footer_td1.appendChild( button );
        
        footer_td2.appendChild(submit);
        footer_tr.appendChild(footer_td1);
        footer_tr.appendChild(footer_td2);
        footer_tbody.appendChild(footer_tr);
        footer_table.appendChild(footer_tbody);
        div_footer.appendChild(footer_table);
        
        div_base.appendChild(div_comment_head);
        div_base.appendChild(div_comment);
        div_base.appendChild(div_tags_head);
        div_base.appendChild(div_tags);
        div_base.appendChild(div_footer);
    	
    	return div_base;
    }
    
    //-----------------------------------------------------
    function makeButton(id, value, func) {
        var submit = document.createElement('input');
        var p      = document.createElement('p');
        var event  = setHandler(id, submit, bindFunc(submit, func));
        
        submit.className = 'submit';
        submit.setAttribute('type',  'button');
        submit.setAttribute('value', value);
        submit.setAttribute('id', id);
        submit.attachEvent('onclick', event );
        
        p.appendChild(submit);
        return p;
    }
    
    //-----------------------------------------------------
    function makeDotLine() {
        var tr  = document.createElement('tr');
        var td  = document.createElement('td');
        var img = document.createElement('img');
        
        img.src = 'http://res.nicovideo.jp/img/_.gif';
        img.setAttribute('height', '1');
        img.setAttribute('width',  '1');
        
        td.className = 'dot_1';
        td.colSpan = 4;
        td.appendChild(img);
        tr.appendChild(td);
        
        return tr;
    }
    
    //-----------------------------------------------------
    function makePagenation(list) {
        //
        var showLink     = 10;
        var showLinkLeft = parseInt(showLink / 2);
        var pos          = mylist.pos;
        
        var pageCurrent = Math.ceil(mylist.pos / NUM_DISPALY);
        var pageBegin   = (pageCurrent - showLinkLeft < 0) ? 0 : (pageCurrent - showLinkLeft);
        var pageMax     = parseInt((list.length - 1) / NUM_DISPALY) + 1;
        var pageEnd     = ((pageBegin + showLink) < pageMax)? (pageBegin + showLink) : pageMax;
        
        if (pageBegin > (pageEnd - showLink)) {
        	pageBegin = ((pageEnd - showLink) < 0)? 0 : (pageEnd - showLink);
        }
        
        var offsetPrev = ((pos - NUM_DISPALY) < 0)? 0 : (pos - NUM_DISPALY);
        var offsetNext = pos + NUM_DISPALY;
        
        //
        var div = document.createElement('div');
        
        div.appendChild(makeNavigateAllow(offsetPrev, list.length, true));
        div.appendChild(makePaginationList(pageCurrent, pageBegin, pageEnd));
        div.appendChild(makeNavigateAllow(offsetNext, list.length, false));
        
        div.className = 'pagelink';
        div.style.cssText = 'text-align: right; line-height:1.6;';
        
        return div;
    }
    
    //-----------------------------------------------------
    function makeNavigateAllow(offset, length, prev) {
        var obj;
        
        if (prev) {
            if (mylist.pos > 0) {
                obj = makeActiveLink(offset, '&lt;&nbsp;߂');
            } else {
                obj = makeNegativeLink('< ߂');
                obj.className = 'prevpage';
            }
        } else {
            if ((mylist.pos + NUM_DISPALY) < length) {
                obj = makeActiveLink(offset, 'i&nbsp;&gt;');
            } else {
                obj = makeNegativeLink('i >');
                obj.className = 'nextpage';
            }
        }
        
        return obj;
    }
    
    function makeActiveLink(offset, name) {
        var link  = document.createElement('a');
        var event = setHandler('linkto_'+ offset, link, getActiveLinkEventHandler(offset));
        
        link.href = 'javascript:void(0);';
        link.innerHTML = name;
        link.attachEvent('onclick', event);
        
        return link;
    }
    
    function getActiveLinkEventHandler(offset) {
        return function() {
            mylist.pos = offset;
            updateList();
            setReloadTimer();
        }
    }
    
    function makeNegativeLink(name) {
        var span = document.createElement('span');
        var text = document.createTextNode(name);
        
        span.appendChild(text);
        return span;
    }
    
    //-----------------------------------------------------
    function makePaginationList(curr, begin, end) {
        var base = document.createElement('span');
        var pos  = curr;
        var offset;
        base.style.cssText = 'margin:0; padding:0; border:none';
        
        for (var i=begin; i<end; ++i) {
            offset = mylist.pos + (i - curr) * NUM_DISPALY;
            offset = (offset < 0)? 0 : offset;
            
            if (i == curr) {
                var span = document.createElement('span');
                
                span.innerHTML = (i + 1);
                span.className = 'in';
                base.appendChild(span);
            } else {
                var a = document.createElement('a');
                var event = setHandler('paginate_'+ offset, a, getActiveLinkEventHandler(offset));
                
                a.innerHTML = (i + 1);
                a.setAttribute('href',  'javascript:void(0);');
                base.appendChild(a);
                
                a.attachEvent('onclick', event);
            }
        }
        
        return base;
    }
    
    //-----------------------------------------------------
    function updateItemCount(count) {
    	var mylist_items_length = document.getElementById('mylist_items_length');
        if (mylist_items_length) {
        	mylist_items_length.innerHTML = count;
        }
    }
    
    //-----------------------------------------------------
    function updateTagLink(item) {
    	var target = document.getElementById('am_tags_'+ item.id);
        var len    = item.tags.length;
        var str;
        
        removeAllChild(target);
        
        for (var i=0; i<len; ++i) {
        	str = mylist.getTagName(item.tags[i]);
        	if (str) {
        		var a = document.createElement('a');
		    	var t = item.tags[i];
                var k = item.id +"_"+ item.tags[i];
                var e = setHandler(k, a, getTagEventHandler(item.tags[i]));
		    	
		    	a.innerHTML = escapeString(str);
		    	a.style.cssText = 'font-size:11px';
		    	a.setAttribute('href',  'javascript:void(0)');
		    	a.attachEvent('onclick', e);
                
		    	target.appendChild(a);
		    	target.appendChild(document.createTextNode(' '));
        	}
        }
    }
    
    function getTagEventHandler(id) {
        return function() {
            _change_tag_click(id);
        }
    }
    
    //-----------------------------------------------------
    function updateTagList(id, item) {
        var target = document.getElementById(id);
        var vid    = id.substring( id.lastIndexOf('_') + 1 );
        var len    = item.tags.length;
        
        removeAllChild(target);
        
        if (len > 0) {
        	for (var i=0; i<len; ++i) {
        		var tid   = item.tags[i];
        		var tname = mylist.getTagName(tid);
	        	var tr  = document.createElement('tr');
	        	var td1 = document.createElement('td');
	        	var td2 = document.createElement('td');
	        	var del = document.createElement('input');
	        	
	        	del.setAttribute('id',   'am_delete_tag_'+ vid +'_'+ tid);
	        	del.setAttribute('type', 'button');
	        	del.setAttribute('value', '폜');
	        	del.className = 'submit';
	        	td1.className = 'TXT12';
	        	td2.className = 'TXT12';
	        	td1.style.cssText = 'border-bottom:dotted 1px #666; padding: 2px 3em 0 4px;';
	        	td2.style.cssText = 'border-bottom:dotted 1px #666;';
	        	
	        	del.attachEvent('onclick', bindFunc(del, _delete_tag) );
	        	
	        	td1.appendChild( document.createTextNode(' '+ tname) );
	        	td2.appendChild(del);
	        	tr.appendChild(td1);
	        	tr.appendChild(td2);
	        	target.appendChild(tr);
        	}
        } else {
        	var tr = document.createElement('tr');
        	var td = document.createElement('td');
        	
        	td.className = 'TXT12';
        	td.style.cssText = 'border-bottom:dotted 1px #666; ';
        	
        	td.appendChild( document.createTextNode('o^Ă^O͂܂') );
        	tr.appendChild(td);
        	target.appendChild(tr);
        }
    }
    //-----------------------------------------------------
    function updateTagSelectBox(id, setbase) {
    	var target = document.getElementById(id);
    	var vid    = id.substring( id.lastIndexOf('_') + 1 );
    	
        removeAllChild(target);
        
        if (setbase) {
        	var base = document.createElement('option');
	    	base.value = '-1';
	    	base.innerHTML = 'ׂĕ\';
	    	base.selected  = (mylist.showtag == 0);
	    	target.appendChild(base);
    	} else {
    		if (mylist.tags.length == 0) {
    			target.setAttribute('disable', true);
    		}
    	}
    	
    	var len = mylist.tags.length;
    	for (var i=0; i<len; ++i) {
    		var option = document.createElement('option');
    		option.value     = mylist.tags[i].id;
    		option.innerHTML = mylist.tags[i].t;
    		option.selected  = ((i+1) == mylist.showtag);
    		target.appendChild(option);
    	}
    }
    
    //-----------------------------------------------------
    function enableButtons() {
        var b = document.getElementsByTagName('input');
        for (var i=0; i<b.length; ++i) {
            if (b[i].id.indexOf("am_edit_") == 0 || b[i].id.indexOf("am_del_") == 0) {
                b[i].removeAttribute('disabled');
            }
        }
    }
    function disableButtons() {
        var b = document.getElementsByTagName('input');
        for (var i=0; i<b.length; ++i) {
            if (b[i].id.indexOf("am_edit_") == 0 || b[i].id.indexOf("am_del_") == 0) {
                b[i].setAttribute('disabled', true);
            }
        }
    }
    
    
    //===============================================================
    // Event Handler
    //===============================================================
    function _begin_edit() {
    	var id   = this.id.substring( this.id.lastIndexOf('_') + 1 );
        var idx  = mylist.findItem(id);
        var form = document.getElementById('am_editform_'+ id);
        
        if (!form) {
        	var am_desc = document.getElementById('am_desc_'+ id);
        	if (am_desc) {
        		am_desc.parentNode.insertBefore(makeEditPanel(mylist.items[idx]), am_desc.nextSibling);
        		form = document.getElementById('am_editform_'+ id);
        	}
        }
        
        if (form) {
        	w.clearInterval(tm);            // stop reload timer
        	tm = null;
        	
        	updateTagList('am_edit_tags_'+id, mylist.items[idx]);
        	disableButtons();               // disable actions
	        form.style.display = 'block';	// open edit form
	        
	        document.getElementById('am_append_tag_edit_'+ id).focus();
        }
    };
    
    function _end_edit() {
        var id   = this.id.substring( this.id.lastIndexOf('_') + 1 );
        var form = document.getElementById('am_editform_'+ id);
        
        if (form) {
        	var idx = mylist.findItem(id);
        	var cm  = document.getElementById('am_edit_comment_'+ id);
        	var dsc = document.getElementById('am_desc_'+ id);
        	
        	if ((idx !== false) && cm && dsc) {
                mylist.items[idx].c = Punycode.encode(cm.value);
        		mylist.saveList();
        		
        		dsc.innerHTML = escapeString(cm.value);
        	}
        	
        	form.style.display = 'none';		// close edit form
        	setReloadTimer();					// start reload timer
        	enableButtons();					// enable actions
        	updateTagLink(mylist.items[idx]);	// update tag link
        }
    };
    
    function _delete_item(id) {
        var id  = this.id.substring( this.id.lastIndexOf('_') + 1 );
        var idx = mylist.findItem(id);
        
        if (w.confirm('{ɍ폜܂H')) {
            if (mylist.deleteItem(mylist.items[idx])) {
                updateList();
            } else {
                //GM_log('delete item failed ['+ id +']');
            }
        }
    };
    
    function _add_tag() {
    	var id   = this.id.substring( this.id.lastIndexOf('_') + 1 );
    	var idx  = mylist.findItem(id);
    	var edit = document.getElementById('am_append_tag_edit_'+ id);
    	var tag  = edit.value;
    	
    	if (mylist.addItemTag(id, tag)) {
    		updateTagList('am_edit_tags_'+ id, mylist.items[idx]);
    		updateTagSelectBox('mylist_select_tag', true);
    	} else {
    		GM_log('add item tag failed.');
    	}
    	edit.value = '';
    	edit.focus();
    };
    
    function _delete_tag() {
    	var pos_tid = this.id.lastIndexOf('_');
    	var pos_id  = this.id.lastIndexOf('_', (pos_tid - 1));
    	
    	var tid = this.id.substring( pos_tid + 1 );
    	var id  = this.id.substring( pos_id  + 1, pos_tid );
    	var idx = mylist.findItem(id);
    	
    	if (mylist.deleteItemTag(id, tid)) {
    		updateTagList('am_edit_tags_'+ id, mylist.items[idx]);
    		updateTagSelectBox('mylist_select_tag', true);
    	} else {
    		GM_log("deleteItemTag() failed.");
    	}
    };
    
    function _change_tag() {
    	var t = document.getElementById('mylist_select_tag');
    	if (t) {
	    	mylist.pos = 0;
	    	mylist.showtag = t.selectedIndex;
	    	updateList();
	    	setReloadTimer();
	    }
    };
    function _change_tag_click(id) {
    	var t = document.getElementById('mylist_select_tag');
    	if (t) {
    		var len = t.options.length;
    		for (var i=0; i<len; ++i) {
    			t.options[i].selected = (id == t.options[i].value);
    		}
    	}
    	_change_tag();
    };
    
    //===============================================================
    // Others
    //===============================================================
    function getVideoInfo() {
        var id = '';	// id (watch/XXXXXX)
        var v  = '';	// id (smXXXXXX)
        var t  = '';	// title
        
        var m = w.location.href.match(/^http:\/\/.*?\.nicovideo\.jp\/watch\/([^\/]+)/);
        if (m) {
            id = m[1];
        }
        
        t = Punycode.encode(getVideoTitle() || getVideoTitle(true));
        v = getVideoId(id);
        if (!id || !t) return false;
        
        return new ListItem(id, v, t);
    }
    
    function getVideoTitle(lenient) {
        var h1 = document.getElementsByTagName('h1')[0];
        if (h1) {
            var a = h1.getElementsByTagName('a');
            for (var i=0; i<a.length; ++i) {
                if (a[i].className == 'video' || lenient) {
                    return a[i].innerHTML;
                }
            }
            var inp = h1.getElementsByTagName('input');
            for (var i=0; i<inp.length; ++i) {
                if (inp[i].name == 'm_title' || lenient) {
                    return inp[i].title;
                }
            }
        }
        return false;
    }
    
    function getVideoId(id) {
        var div = document.getElementById('video_tags');
        var a   = div.getElementsByTagName('a');
        var m;
        
        for (var i=0; i<a.length; ++i) {
            m = a[i].href.match(/http:\/\/.*?\.nicovideo\.jp\/tag_edit\/([a-z0-9]+)/);
            if (m && (m[1] != id)) {
                return v;
            }
        }
        return false;
    }
    
    function getDateFormat(date) {
        var _date = Array();
        _date['year']  = date.getYear();
        _date['month'] = date.getMonth() + 1;
        _date['date']  = date.getDate();
        _date['hour']  = date.getHours();
        _date['min']   = date.getMinutes();
        _date['sec']   = date.getSeconds();
        
        for (var key in _date) {
            if (key != 'year') {
                _date[key]  = (_date[key] < 10)? ('0'+ _date[key]) : _date[key];
            } else {
                _date[key]  = (_date[key] < 2000)? (_date[key] + 1900) : _date[key];
            }
        }
        
        return _date['year']  +'N'
             + _date['month'] +''
             + _date['date']  +' '
             + _date['hour']  +'F'
             + _date['min']   +'F'
             + _date['sec'];
    }
    
    function removeAllChild(target) {
    	if (!target) return;
    	
        if (target.firstChild) {
            while(target.firstChild) {
                target.removeChild(target.firstChild);
            }
        }
    }
    
    function setHandler(id, obj, func) {
        handler[id] = new Array(obj, func);
        
        return handler[id][1];
    }
    
    function detachHandler(id, name) {
        if (handler[id] != null) {
            handler[id][0].detachEvent(name, handler[id][1]);
            handler[id] = null;
        }
    }
    
    function escapeString(str) {
    	return str.replace(/&/,'&amp;').replace(/</g,'&lt;').replace(/>/,'&gt;');
    }
    
    //===============================================================
    // Greasemonkey Methods
    //===============================================================
    db.Query("CREATE TABLE IF NOT EXISTS mylist (name TEXT primary key, value);");
    
    function GM_getValue(name) {
        var result = db.Query("SELECT value FROM mylist WHERE name = '"+ name +"';" );
        
        if (result.NumRows()) {
            return result.FetchArray().At(0);
        }
        return;     // undefined
    }
    
    function GM_setValue(name, value) {
        value = value.replace(/'/g, "''");
        var result = db.Query("SELECT value FROM mylist WHERE name = '"+ name +"';" );
        
        if (result.NumRows()) {
            db.Query("UPDATE mylist SET value = '"+ value +"' WHERE name = '"+ name +"';" );
        } else {
            db.Query("INSERT INTO mylist (name, value) VALUES ('"+ name +"', '"+ value +"');" );
        }
    }
    
    //===============================================================
    // uneval() - written by Dan Kogai
    // http://blog.livedoor.jp/dankogai/archives/50958256.html
    //===============================================================
    var protos = [];
    var uneval_asis = function(o){ return o.toString() };
    
    /* predefine objects where typeof(o) != 'object' */
    var name2uneval = {
        'boolean':uneval_asis,
        'number': uneval_asis,
        'string': function(o){
            return '\'' 
            + o.toString().replace(/[\\\"\']/gm, function(m0){
                return '\\' + m0;
            }).replace(/\r\n|\n/gm, "\\n")
            + '\'';
        },
        'undefined': function(o){ return 'undefined' },
        'function':uneval_asis
    };
    
    var uneval_default = function(o, np){
        var src = []; // a-ha!
        for (var p in o){
            if (!o.hasOwnProperty(p)) continue;
            src[src.length] = uneval(p)  + ':' + uneval(o[p], 1);
        }
        // parens needed to make eval() happy
        return np ? '{' + src.toString() + '}' : '({' + src.toString() + '})';
    };
    
    uneval_set = function(proto, name, func){
        protos[protos.length] = [ proto, name ];
        name2uneval[name] = func || uneval_default;
    };
    
    uneval_set(Array, 'array', function(o){
        var src = [];
        for (var i = 0, l = o.length; i < l; i++)
            src[i] = uneval(o[i]);
        return '[' + src.toString() + ']';
    });
    uneval_set(RegExp, 'regexp', uneval_asis);
    uneval_set(Date, 'date', function(o){
        return '(new Date(' + o.valueOf() + '))';
    });
    
    var typeName = function(o){
        // if (o === null) return 'null';
        var t = typeof o;
        if (t != 'object') return t;
        // we have to lenear-search. sigh.
        for (var i = 0, l = protos.length; i < l; i++){
            if (o instanceof  protos[i][0]) return protos[i][1];
        }
        return 'object';
    };
    
    uneval = function(o, np){
        // if (o.toSource) return o.toSource();
        if (o === null) return 'null';
        var func = name2uneval[typeName(o)] || uneval_default;
        return func(o, np);
    }
    
    //===============================================================
    // 
    //===============================================================
    function bindFunc(bind, func){
        return function(){
            return func.apply(bind, arguments);
        };
    }
    
    //===============================================================
    // main
    //===============================================================
    new Punycode;
    var mylist = new MyList();
    
    if (String(document.location).indexOf('.nicovideo.jp/watch') !=  - 1) {
        screenWatch();
    } else {
        screenMylist();
    }
    
    
    //===============================================================
    function Punycode()
    {
      Punycode.encode = function(input){ return input; };
      Punycode.decode = function(input){ return input; };
    }

};
